# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv, fields
from openerp.tools import DEFAULT_SERVER_DATE_FORMAT
from openerp import netsvc
import time


class sale_order(osv.Model):

    """Extend the sale.order model to meet AXIMA requirements"""
    _inherit = 'sale.order'

    def _get_shop_id(self, cr, uid, id, default=None, context=None):
        """Renvoie le numéro d'identification du magasin rattaché à l'agence à
        laquelle est associé l'utilisateur courant."""
        user = self.pool.get('res.users').browse(cr, uid, uid,
                                                 context=context)
        company_id = user.company_id.id
        company_name = user.company_id.name
        shop_ids = self.pool.get('sale.shop').search(
            cr, uid, [('company_id', '=', company_id)], context=context)

        if shop_ids:
            return shop_ids[0]

        raise osv.except_osv(
            _("Error"),
            _('No shop defined for the company "%s"' % company_name)
        )

    def copy(self, cr, uid, id, default=None, context=None):
        """Modifie la copie de façon à ce que les différentes agences puissent
        dupliquer tous les devis, y compris ceux créés initialement par
        d'autres agences.

        Par défaut les autres agences ne peuvent pas dupliquer les devis des
        autres agences du fait de restrictions d'écriture.
        """
        user = self.pool.get('res.users').browse(cr, uid, uid,
                                                 context=context)
        shop_id = self._get_shop_id(cr, uid, id, default, context)

        if not default:
            default = {}
        default.update({
            # affecté au commercial connecté qui duplique (et à sa company)
            'user_id': uid,  
            'company_id': user.company_id and user.company_id.id or False,
            
            # reinit client/contact/signatures
            'contact': False,
            'signature1': False,
            'signature2': False,
            
            'date_order': fields.date.context_today(self, cr, uid,
                                                    context=context),
            'state': 'draft',
            'invoice_ids': [],
            'date_confirm': False,
            'sap_order_ref': '',
            'client_order_ref': '',
            'name': self.pool.get('ir.sequence').get(cr, uid, 'sale.order'),
            'shop_id': shop_id,
        })
        return super(
            sale_order,
            self).copy(
            cr,
            uid,
            id,
            default,
            context=context)

    def _invoiced_rate(self, cursor, user, ids, name, arg, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' près.
        """
        res = {}
        for sale in self.browse(cursor, user, ids, context=context):
            if sale.invoiced:
                res[sale.id] = 100.0
                continue
            tot = 0.0
            for invoice in sale.invoice_ids:
                if invoice.state not in ('draft', 'cancel', 'refuse'):
                    tot += invoice.amount_untaxed
            if tot:
                res[sale.id] = min(
                    100.0, tot * 100.0 / (sale.amount_untaxed or 1.00))
            else:
                res[sale.id] = 0.0
        return res

    def _invoiced(self, cursor, user, ids, name, arg, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' près."""
        res = {}
        for sale in self.browse(cursor, user, ids, context=context):
            res[sale.id] = True
            invoice_existence = False
            for invoice in sale.invoice_ids:
                if invoice.state not in ('cancel', 'refuse'):
                    invoice_existence = True
                    if invoice.state != 'paid':
                        res[sale.id] = False
                        break
            if not invoice_existence or sale.state == 'manual':
                res[sale.id] = False
        return res

    def _invoiced_search(self, cursor, user, obj, name, args, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' et un formatage des lignes près.
        """
        if not len(args):
            return []
        clause = ''
        sale_clause = ''
        no_invoiced = False
        for arg in args:
            if arg[1] == '=':
                if arg[2]:
                    clause += "AND inv.state = 'paid'"
                else:
                    clause += ("AND inv.state != 'cancel' "
                               "AND sale.state != 'cancel' "
                               "AND inv.state != 'refuse' "
                               "AND sale.state != 'refuse' "
                               "AND inv.state <> 'paid'  "
                               "AND rel.order_id = sale.id ")
                    sale_clause = ',  sale_order AS sale '
                    no_invoiced = True

        select_query = (
            'SELECT rel.order_id '
            'FROM sale_order_invoice_rel AS rel, account_invoice AS inv {0}'
            'WHERE rel.invoice_id = inv.id {1}').format(
            sale_clause,
            clause)
        cursor.execute(select_query)
        res = cursor.fetchall()
        if no_invoiced:
            select_query = ("SELECT sale.id FROM sale_order AS sale "
                            "WHERE sale.id NOT IN "
                            "(SELECT rel.order_id "
                            "FROM sale_order_invoice_rel AS rel) "
                            "and sale.state != 'cancel'"
                            "and sale.state != 'refuse'")
            cursor.execute(select_query)
            res.extend(cursor.fetchall())
        if not res:
            return [('id', '=', 0)]
        return [('id', 'in', [x[0] for x in res])]
        
    def _get_fake(self, cr, uid, ids, name, args, context=None):
        res = {}
        for id in ids:
            res[id] = False
        return res
        
    def _user_of_company_search(self, cr, uid, ids, name, args, context=None):
        if not len(args):
            return [('id', '=', 0)]
        r_user = self.pool.get('res.users').read(cr, uid, uid, ['company_id'],
            context=context)
        if not r_user or not r_user['company_id']:
            return [('id', '=', 0)]
        domain = [('company_id', '=', r_user['company_id'][0])]
        ids = self.search(cr, uid, domain, context=context)
        if not ids:
            return [('id', '=', 0)]
        return [('id', 'in', ids)]

    _state_help = ("Gives the status of the quotation or sales order. \n"
                   "The exception status is automatically set when a cancel "
                   "operation occurs in the processing of a document linked "
                   "to the sales order. \nThe 'Waiting Schedule' status is "
                   "set when the invoice is confirmed but waiting for the "
                   "scheduler to run on the order date.")
                   
    _columns = {
        'object': fields.char(u"Objet",
                              size=64,
                              required=False),

        'internal_order_ref': fields.char(u"Nos références",
                                          size=64,
                                          required=False),

        'sap_order_ref': fields.char(u"Nᵒ devis",
                                     size=64,
                                     required=False),

        'final_recipient': fields.char(u"Destinataire final",
                                       size=64,
                                       required=False, ),

        'expiration_date': fields.date(u"Date d’expiration",
                                       required=False, ),

        'supply_period': fields.char(u"Délai d’approvisionnement",
                                     size=64,
                                     required=True),

        'execution_period': fields.char(u"Délai d’exécution",
                                        size=64,
                                        required=True),

        'contact': fields.many2one(
            'res.partner',
            u"Chargé de contact",
            required=False),

        'signature1': fields.many2one(
            'res.partner',
            u"Signature commerciale",
            required=False),

        'signature2': fields.many2one(
            'res.partner',
            u"Seconde signature commerciale",
            required=False),

        'client_note': fields.related(
            'partner_id',
            'comment',
            string='Notes internes',
            type="text",
            store=False,
            ),

        'commercial_note': fields.related(
            'partner_id',
            'commercial_note',
            string='Conditions commerciales',
            type="text",
            store=False,
            ),

        'extra_note': fields.text(
            u'Informations supplémentaires à ajouter face aux totaux',
            required=False
            ),
        # Modification du libelé de l'état manual de "commande à facturer"  en
        # "Devis accepté" et ajout de l'état refuse
        'state': fields.selection([
            ('draft', 'Draft Quotation'),
            ('sent', 'Quotation Sent'),
            ('cancel', 'Cancelled'),
            ('refuse', u'Refusé'),
            ('waiting_date', 'Waiting Schedule'),
            ('progress', 'Sales Order'),
            ('manual', 'Devis accepté'),
            ('invoice_except', 'Invoice Exception'),
            ('done', 'Done'),
            ], 'Status', readonly=True, track_visibility='onchange',
            help=_state_help,
            select=True),
            
        # my company quotes filter
        'user_of_company': fields.function(_get_fake, type='boolean',
            fnct_search= _user_of_company_search,
            string="Utilisateur de l'agence", method=True),
            
        # show discount ?
        'show_discount': fields.boolean('Remise (ligne) visible sur le devis ?'),
        
        # saisie forfaitisée
        # report: affiche d'une ligne virtuelle
        # qté 1, price_unit = amount_untaxed (montant total ht),
        # désignation = désignation du forfait
        'set_price_on': fields.boolean('Devis forfaitisé ?',
            help="Si activé, à l'édition seule une ligne va représenter le forfait (détail des articles non visible)"),
        'set_price_name': fields.text('Désignation (du forfait)', help="Désignation de la ligne du forfait"),
    }
    
    _defaults = {
        'show_discount': False,
        'set_price_on': False,
    }

    def onchange_partner_id(
            self,
            cr,
            uid,
            ids,
            partner_id,
            client_note,
            commercial_note,
            context):
        """Update client_note and commercial_note with those of the partner_id
        """
        notes_on_partner = {}
        if partner_id:
            partner = self.pool.get('res.partner').browse(cr, uid, partner_id)
            notes_on_partner['client_note'] = partner.comment
            notes_on_partner['commercial_note'] = partner.commercial_note

        inherited_values = super(
            sale_order,
            self).onchange_partner_id(
            cr,
            uid,
            ids,
            partner_id,
            context)
        value_entry = dict(inherited_values['value'].items()
                           + notes_on_partner.items())
        inherited_values['value'] = value_entry
        return inherited_values

    # Form filling
    def unlink(self, cr, uid, ids, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' et un formatage des lignes près.
        """
        sale_orders = self.read(cr, uid, ids, ['state'], context=context)
        unlink_ids = []
        for s in sale_orders:
            if s['state'] in ['draft', 'cancel', 'refuse']:
                unlink_ids.append(s['id'])
            else:
                msg = ('In order to delete a confirmed sales order, '
                       'you must cancel it before!')
                raise osv.except_osv(_('Invalid Action!'), _(msg))

        return osv.osv.unlink(self, cr, uid, unlink_ids, context=context)

    def _make_invoice(self, cr, uid, order, lines, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' et un formatage des lignes près.
        """
        inv_obj = self.pool.get('account.invoice')
        obj_invoice_line = self.pool.get('account.invoice.line')
        if context is None:
            context = {}
        invoiced_sale_line_ids = self.pool.get('sale.order.line').search(
            cr, uid, [('order_id', '=', order.id), ('invoiced', '=', True)],
            context=context)
        from_line_invoice_ids = []
        for invoiced_sale_line_id in self.pool.get('sale.order.line').browse(
                cr, uid, invoiced_sale_line_ids, context=context):
            for invoice_line_id in invoiced_sale_line_id.invoice_lines:
                if invoice_line_id.invoice_id.id not in from_line_invoice_ids:
                    from_line_invoice_ids.append(invoice_line_id.invoice_id.id)
        for preinv in order.invoice_ids:
            if (preinv.state not in ('cancel', 'refuse') and
                    preinv.id not in from_line_invoice_ids):
                for preline in preinv.invoice_line:
                    inv_line_id = obj_invoice_line.copy(
                        cr, uid, preline.id, {
                            'invoice_id': False,
                            'price_unit': -preline.price_unit})
                    lines.append(inv_line_id)
        inv = self._prepare_invoice(cr, uid, order, lines, context=context)
        inv_id = inv_obj.create(cr, uid, inv, context=context)
        data = inv_obj.onchange_payment_term_date_invoice(
            cr,
            uid,
            [inv_id],
            inv['payment_term'],
            time.strftime(DEFAULT_SERVER_DATE_FORMAT))
        if data.get('value', False):
            inv_obj.write(cr, uid, [inv_id], data['value'], context=context)
        inv_obj.button_compute(cr, uid, [inv_id])
        return inv_id

    def action_invoice_refuse(self, cr, uid, ids, context=None):
        """Adaptation d'une Copie de la méthode action_invoice_cancel de
        sale_order.
        """
        self.write(cr, uid, ids, {'state': 'invoice_except'}, context=context)
        return True

    def action_cancel(self, cr, uid, ids, context=None):
        """Copie exacte de la méthode originale de sale_order, à la prise en
        compte de l'état 'refuse' et un formatage des lignes près.
        """
        wf_service = netsvc.LocalService("workflow")
        if context is None:
            context = {}
        sale_order_line_obj = self.pool.get('sale.order.line')
        for sale in self.browse(cr, uid, ids, context=context):
            for inv in sale.invoice_ids:
                if inv.state not in ('draft', 'cancel', 'refuse'):
                    msg = ('First cancel all invoices attached to this sales '
                           'order.')
                    raise osv.except_osv(
                        _('Cannot cancel this sales order!'),
                        _(msg))
            for r in self.read(cr, uid, ids, ['invoice_ids']):
                for inv in r['invoice_ids']:
                    wf_service.trg_validate(uid, 'account.invoice', inv,
                                            'invoice_cancel', cr)
            sale_order_line_obj.write(cr, uid, [l.id for l in sale.order_line],
                                      {'state': 'cancel'})
        self.write(cr, uid, ids, {'state': 'cancel'})
        return True

    def action_refuse(self, cr, uid, ids, context=None):
        """Adaptation d'une Copie de la méthode action_cancel de sale_order."""
        wf_service = netsvc.LocalService("workflow")
        if context is None:
            context = {}
        sale_order_line_obj = self.pool.get('sale.order.line')
        for sale in self.browse(cr, uid, ids, context=context):
            for inv in sale.invoice_ids:
                if inv.state not in ('draft', 'cancel', 'refuse'):
                    msg = ('First cancel all invoices attached to this sales '
                           'order.')
                    raise osv.except_osv(
                        _('Cannot cancel this sales order!'),
                        _(msg))
            for r in self.read(cr, uid, ids, ['invoice_ids']):
                for inv in r['invoice_ids']:
                    wf_service.trg_validate(uid, 'account.invoice', inv,
                                            'invoice_refuse', cr)
            sale_order_line_obj.write(cr, uid, [l.id for l in sale.order_line],
                                      {'state': 'refuse'})
        self.write(cr, uid, ids, {'state': 'refuse'})
        return True
       
    def onchange_order_line(self, cr, uid, ids, order_line):
        """
        AX-23: delete cascade of products childs of a pack
        order_line:
        [[1, id, {'field_1': ,}], [2, id, False], ]
        1: update, 2 delete: see orm.py/write
        """  
        res = {}
        sol_to_update_ids = []
        sol_to_delete_ids = []
        for record in order_line:
            if len(record) == 3:
                if record[0] == 1:
                    # update
                    pass
                elif record[0] == 2:
                    # delete
                    sol = self.pool.get('sale.order.line').browse(cr, uid,
                        record[1])
                    self._onchange_order_line_get_childs_ids(sol,
                        sol_to_delete_ids)                              
        
        if sol_to_update_ids or sol_to_delete_ids:
            res_order_line_value = []
            for record in order_line:
                if len(record) == 3:
                    if record[1] in sol_to_update_ids \
                        and record[1] not in sol_to_delete_ids:
                        # a delete status from delete pack cascade is prior
                        # than a line update
                        res_order_line_value.append(record)
                    elif record[1] in sol_to_delete_ids:
                        res_order_line_value.append([2, record[1], False])
                    else:
                        res_order_line_value.append(record)
            res = {'value': {'order_line': res_order_line_value}}
        return res
        
    def _onchange_order_line_get_childs_ids(self, sol, res_ids, depth=1):
        if depth >= 10:
            return  # nan_product_module has a depth limit of 10
        if sol.pack_child_line_ids:
            for child_sol in sol.pack_child_line_ids:
                if child_sol.id not in res_ids:
                     res_ids.append(child_sol.id)
                self._onchange_order_line_get_childs_ids(child_sol,
                    res_ids, depth + 1)
                    
   
    def hook_nan_product_pack_expand_packs_order_line_vals(self, cr, uid, ids,
        order, product, vals, context=None):
        """
        :type order: browse object sale.order
        :type product: browse object product.product
        """
        purchase_price = product.standard_price
        frm_cur = self.pool.get('res.users').browse(cr, uid, uid).company_id.currency_id.id
        to_cur = order.pricelist_id.currency_id.id
        price = self.pool.get('res.currency').compute(cr, uid, frm_cur, to_cur, purchase_price, round=False)
        if price:
            vals['purchase_price'] = price
                        
    """
     OVERLOAD: expand_packs nan_product_pack/sale.py
     note: use above hook(s)
    """
    def create(self, cr, uid, vals, context=None):
        result = super(sale_order,self).create(cr, uid, vals, context)
        self.expand_packs(cr, uid, [result], context)
        return result

    def write(self, cr, uid, ids, vals, context=None):
        result = super(sale_order,self).write(cr, uid, ids, vals, context)
        self.expand_packs(cr, uid, ids, context)
        return result
    
    def expand_packs(self, cr, uid, ids, context={}, depth=1):
        # OVERLOAD: expand_packs nan_product_pack/sale.py

        def get_real_price(res_dict, product_id, qty, uom, pricelist):
            item_obj = self.pool.get('product.pricelist.item')
            price_type_obj = self.pool.get('product.price.type')
            product_obj = self.pool.get('product.product')
            template_obj = self.pool.get('product.template')
            field_name = 'list_price'

            if res_dict.get('item_id',False) and res_dict['item_id'].get(pricelist,False):
                item = res_dict['item_id'].get(pricelist,False)
                item_base = item_obj.read(cr, uid, [item], ['base'])[0]['base']
                if item_base > 0:
                    field_name = price_type_obj.browse(cr, uid, item_base).field

            product = product_obj.browse(cr, uid, product_id, context)
            product_tmpl_id = product.product_tmpl_id.id

            product_read = template_obj.read(cr, uid, product_tmpl_id, [field_name], context)

            factor = 1.0
            if uom and uom != product.uom_id.id:
                product_uom_obj = self.pool.get('product.uom')
                uom_data = product_uom_obj.browse(cr, uid,  product.uom_id.id)
                factor = uom_data.factor
            return product_read[field_name] * factor

        if depth == 10:
            return
        updated_orders = []

        for order in self.browse(cr, uid, ids, context):
            
            fiscal_position = order.fiscal_position and self.pool.get('account.fiscal.position').browse(cr, uid, order.fiscal_position, context) or False

            # The reorder variable is used to ensure lines of the same pack go right after their 
            # parent.
            # What the algorithm does is check if the previous item had children. As children items 
            # must go right after the parent if the line we're evaluating doesn't have a parent it
            # means it's a new item (and probably has the default 10 sequence number - unless the 
            # appropiate c2c_sale_sequence module is installed). In this case we mark the item for
            # reordering and evaluate the next one. Note that as the item is not evaluated and it might
            # have to be expanded it's put on the queue for another iteration (it's simple and works well).
            # Once the next item has been evaluated the sequence of the item marked for reordering is updated
            # with the next value.

            sequence = -1
            reorder = []
            last_had_children = False
            for line in order.order_line:
                if last_had_children and not line.pack_parent_line_id:
                    reorder.append( line.id )
                    if line.product_id.pack_line_ids and not order.id in updated_orders:
                        updated_orders.append( order.id )
                    continue

                sequence += 1

                if sequence > line.sequence:
                    self.pool.get('sale.order.line').write(cr, uid, [line.id], {
                        'sequence': sequence,
                    }, context)
                else:
                    sequence = line.sequence

                if line.state != 'draft':
                    continue
                if not line.product_id:
                    continue

                # If pack was already expanded (in another create/write operation or in 
                # a previous iteration) don't do it again.
                if line.pack_child_line_ids:
                    last_had_children = True
                    continue
                last_had_children = False

                for subline in line.product_id.pack_line_ids:
                    sequence += 1

                    subproduct = subline.product_id
                    quantity = subline.quantity * line.product_uom_qty

                    if line.product_id.pack_fixed_price:
                        price = 0.0
                        discount = 0.0
                    else:   
                        pricelist = order.pricelist_id.id
                        list_price = self.pool.get('product.pricelist').price_get(cr, uid, [pricelist], 
                                        subproduct.id, quantity, order.partner_id.id, {
                            'uom': subproduct.uom_id.id,
                            'date': order.date_order,
                        })
                        price = list_price[pricelist]
                        
                        # Added functionality for compatibility with product_visible_discount module
                        pricelist_obj=self.pool.get('product.pricelist')  
                        pricelists = pricelist_obj.read(cr,uid,[pricelist],['visible_discount'])
                        new_list_price = get_real_price(list_price, subproduct.id, quantity, subproduct.uom_id.id, pricelist)
                        discount = line.discount
                        if pricelists[0].has_key("visible_discount"):
                            if(len(pricelists)>0 and pricelists[0]['visible_discount'] and list_price[pricelist] != 0):
                                discount += (new_list_price - price) / new_list_price * 100 
                                price = new_list_price

                    # Obtain product name in partner's language
                    ctx = {'lang': order.partner_id.lang}
                    subproduct_name = self.pool.get('product.product').browse(cr, uid, subproduct.id, ctx).name

                    tax_ids = self.pool.get('account.fiscal.position').map_tax(cr, uid, fiscal_position, subproduct.taxes_id)

                    if subproduct.uos_id:
                        uos_id = subproduct.uos_id.id
                        uos_qty = quantity * subproduct.uos_coeff
                    else:
                        uos_id = False
                        uos_qty = quantity

                    vals = {
                        'order_id': order.id,
                        'name': '%s%s' % ('> '* (line.pack_depth+1), subproduct_name),
                        'sequence': sequence,
                        'delay': subproduct.sale_delay or 0.0,
                        'product_id': subproduct.id,
                        'procurement_id': line.procurement_id and line.procurement_id.id or False,
                        'price_unit': price,
                        'tax_id': [(6,0,tax_ids)],
                        'type': subproduct.procure_method,
                        'property_ids': [(6,0,[])],
                        'address_allotment_id': False,
                        'product_uom_qty': quantity,
                        'product_uom': subproduct.uom_id.id,
                        'product_uos_qty': uos_qty,
                        'product_uos': uos_id,
                        'product_packaging': False,
                        'move_ids': [(6,0,[])],
                        'discount': discount,
                        'number_packages': False,
                        'notes': False,
                        'th_weight': False,
                        'state': 'draft',
                        'pack_parent_line_id': line.id,
                        'pack_depth': line.pack_depth + 1,
                    }

                    # It's a control for the case that the nan_external_prices was installed with the product pack
                    if 'prices_used' in line:
                        vals[ 'prices_used' ] = line.prices_used
                        
                    # Added by TeMPO
                    self.hook_nan_product_pack_expand_packs_order_line_vals(cr,
                        uid, ids, order, subproduct, vals,
                        context=context)

                    self.pool.get('sale.order.line').create(cr, uid, vals, context)
                    if not order.id in updated_orders:
                        updated_orders.append( order.id )

                for id in reorder:
                    sequence += 1
                    self.pool.get('sale.order.line').write(cr, uid, [id], {
                        'sequence': sequence,
                    }, context)

        if updated_orders:
            # Try to expand again all those orders that had a pack in this iteration.
            # This way we support packs inside other packs.
            self.expand_packs(cr, uid, ids, context, depth+1)
        return

    def report_get_price_pack_lines(self, cr, uid, so_br, context=None):
        """
        AX-61
        :return [{'name': ..., 'amount': ...}, ]
        """
        if context is None:
            context = {}
        res = []
        sol_obj = self.pool.get('sale.order.line')

        # temp result: key: parent pack id,
        # value {'name': pack name, 'amount': total amount}
        tmp_res = {}
        packs_total_amount = 0
        for sol in so_br.order_line:
            # key: 0 main products line, other pack products line
            key = sol.pack_parent_line_id.id if sol.pack_parent_line_id else 0
            if not key in tmp_res:
                if key == 0:
                    name = so_br.set_price_name or 'FORFAIT'
                else:
                    name = sol.pack_parent_line_id.product_id and sol.pack_parent_line_id.product_id.name or '',
                    if isinstance(name, (tuple, list, )):
                        name = name[0]
                tmp_res[key] = {'name': name, 'amount': 0., }
            if key != 0 and sol.price_subtotal:
                tmp_res[key]['amount'] += sol.price_subtotal
                packs_total_amount += sol.price_subtotal

        # build res
        # 1st main price line: amount: total untaxed minor packs_total_amount
        res.append({
            'name': tmp_res[0]['name'],
            'amount': (so_br.amount_untaxed or 0.) - packs_total_amount,
        })
        # packs lines
        for pack_id in tmp_res:
            if pack_id != 0:
                res.append({'name': tmp_res[pack_id]['name'],
                    'amount': tmp_res[pack_id]['amount'],
                })
        sorted(res, key=lambda x: name)
        return res

sale_order()


class sale_order_line(osv.Model):

    """Extend the sale.order.line model to meet AXIMA requirements
    """
    _inherit = 'sale.order.line'

    def _comp_discount_price(self, price_unit, discount):
        '''Compute the discount price for given values'''
        return price_unit * (1-(discount or 0.0)/100.0)

    def _unit_discount_price(self, cr, uid, ids, field_name, args,
                             context=None):
        """Return the discount_price of lines
        """
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            res[line.id] = self._comp_discount_price(
                line.price_unit, line.discount)

        return res

    def _fnct_line_invoiced(self, cr, uid, ids, field_name, args,
                            context=None):
        """Copie exacte de la méthode originale de sale_order_line, à la prise
        en compte de l'état 'refuse' et un formatage des lignes près.
        """
        res = dict.fromkeys(ids, False)
        for this in self.browse(cr, uid, ids, context=context):
            res[this.id] = this.invoice_lines and \
                all(iline.invoice_id.state not in ['cancel', 'refuse']
                    for iline in this.invoice_lines)
        return res

    _state_help = ("* The 'Draft' status is set when the related sales "
                   "order in draft status. "
                   "\n* The 'Confirmed' status is set when the related sales "
                   "order is confirmed. "
                   "\n* The 'Exception' status is set when the related sales "
                   "order is set as exception. "
                   "\n* The 'Done' status is set when the sales order line "
                   "has been picked. "
                   "\n* The 'Cancelled' status is set when a user cancel the "
                   "sales order related.")
    _columns = {
        'discount_price': fields.function(_unit_discount_price,
                                          string=u"PU HT net",
                                          ),
        # Ajout de la gestion de l'état 'refuse'
        'state': fields.selection([('cancel', 'Cancelled'),
                                   ('refuse', u'Refusé'),
                                   ('draft', 'Draft'),
                                   ('confirmed', 'Confirmed'),
                                   ('exception', 'Exception'),
                                   ('done', 'Done')], 'Status', required=True,
                                  readonly=True,
                                  help=_state_help),
        }

    def onchange_discount(self, cr, uid, ids, price_unit, discount):
        """"Update the discount price based on given price_unit and discount
        """
        discount_price = self._comp_discount_price(price_unit, discount)

        res = {
            'value': {
                'discount_price':
                discount_price
                },
        }

        if discount > 100.0:
            res['warning'] = {
                'title': u"Erreur : remise trop élevée",
                'message': u"Le prix de vente sera négatif.",
            }
        elif discount < 0.0:
            res['warning'] = {
                'title': u"Attention : remise négative",
                'message': u"Le prix sera suppérieur au prix de base",
            }
        return res

    def button_refuse(self, cr, uid, ids, context=None):
        """Adaptation d'une Copie de la méthode button_cancel de sale_order."""
        for line in self.browse(cr, uid, ids, context=context):
            if line.invoiced:
                msg = ("You cannot cancel a sales order line that has already "
                       "been invoiced.")
                raise osv.except_osv(_('Invalid Action!'), _(msg))
        return self.write(cr, uid, ids, {'state': 'refuse'})

    def unlink(self, cr, uid, ids, context=None):
        """Copie exacte de la méthode originale de sale_order_line, à la prise
        en compte de l'état 'refuse' et un formatage des lignes près.
        """
        if context is None:
            context = {}
        """Allows to delete sales order lines in draft,cancel states"""
        for rec in self.browse(cr, uid, ids, context=context):
            if rec.state and rec.state not in ['draft', 'cancel', 'refuse']:
                msg = ("Cannot delete a sales order line which is in state "
                       "\'%s\'.")
                raise osv.except_osv(
                    _('Invalid Action!'), _(msg) %
                    (rec.state,))
        return super(sale_order_line, self).unlink(cr, uid, ids, context=context)

sale_order_line()


class account_invoice(osv.Model):
    _inherit = 'account.invoice'

    def unlink(self, cr, uid, ids, context=None):
        """Copie exacte de la méthode originale de account_invoice, à la prise
        en compte de l'état 'refuse' et un formatage des lignes près.
        """
        invoice_ids = self.search(cr, uid, [('id', 'in', ids),
                                            ('state', 'in',
                                             ['draft', 'cancel', 'refuse'])],
                                  context=context)
        # if we can't cancel all invoices, do nothing
        if len(invoice_ids) == len(ids):
            # Cancel invoice(s) first before deleting them so that if any sale
            # order is associated with them it will trigger the workflow to put
            # the sale order in an 'invoice exception' state
            wf_service = netsvc.LocalService("workflow")
            for id in ids:
                wf_service.trg_validate(uid, 'account.invoice', id,
                                        'invoice_cancel', cr)
        return super(account_invoice, self).unlink(cr, uid, ids,
                                                   context=context)

account_invoice()

