# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
#from openerp.osv.osv import Model
#from openerp.osv.fields import char, float, integer, selection

from openerp.osv import osv, fields


class res_partner(osv.Model):

    """Extend the res.partner model to meet AXIMA requirements"""
    _inherit = 'res.partner'
    
    def default_get(self, cr, uid, fields, context=None):
        if context is None:
            context = {}
        res = super(res_partner, self).default_get(cr, uid, fields,
            context=context)
            
        if 'search_default_signatories' in context:
            # AX-87: on provient des m2o signataires du sale_order
            # on doit alors créer du contact
            # (par défaut un customer se créé avec 'customer' à True)
            res.update({
                'customer': False,
                'supplier': False,
                'type': 'contact',
            })
            
        return res
    
    def _get_fake(self, cr, uid, ids, name, args, context=None):
        res = {}
        for id in ids:
            res[id] = False
        return res
        
    def _user_of_company_search(self, cr, uid, ids, name, args, context=None):
        if not len(args):
            return [('id', '=', 0)]
        r_user = self.pool.get('res.users').read(cr, uid, uid, ['company_id'],
            context=context)
        if not r_user or not r_user['company_id']:
            return [('id', '=', 0)]
        domain = [('company_id', '=', r_user['company_id'][0])]
        ids = self.search(cr, uid, domain, context=context)
        if not ids:
            return [('id', '=', 0)]
        return [('id', 'in', ids)]
        
    def _get_user_id2(self, cr, uid, ids, name, args, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        res = {}
        for r in self.read(cr, uid, ids, ['name'], context=context):
            res[r['id']] = False
            if r['name']:
                domain = [('name', '=', r['name'])]
                ids = self.pool.get('res.users').search(cr, uid, domain,
                    context=context)
                if ids:
                    res[r['id']] = ids[0]
        return res

    _columns = {
        'sap_client_code': fields.char(u"Code client SAP",
                            required=False, size=10, select=1, ),
        'siret': fields.char(u"SIRET", size=64, required=False),

        'search_criterion': fields.char(u"Critère de recherche",
                                        size=128),

        'name2': fields.char(u"Complément de la raison sociale",
                             size=128,
                             required=False),

        'purchaser': fields.char(u"Donneur d'ordre (DO)",
                                 size=128,
                                 required=False),

        'otp': fields.char(u"OTP", size=128, required=False),

        'tech_post': fields.char(u"Poste technique (PT)", size=128,
                                 required=False),

        'commercial_note': fields.text("Conditions commerciales"),
        
        # for company partner
        'sap_company_code': fields.char(u"Référence SAP", size=10, select=1, ),
        'office_code': fields.char(u"Code agence", size=10, select=1, ),
        
        # for customer
        'country_code': fields.char(u"Code pays",
                            required=False, size=2),
                            
        'user_of_company': fields.function(_get_fake, type='boolean',
            fnct_search= _user_of_company_search,
            string="Utilisateur de l'agence", method=True),
            
        'user_id2': fields.function(_get_user_id2, type='many2one',
            relation='res.users', string="Utilisateur", method=True,
            readonly=True),
    }
    
    def copy(self, cr, uid, id, default=None, context=None):
        """Modifie la copie de façon à ce que les différentes agences puissent
        dupliquer les clients, y compris ceux créés initialement par
        d'autres agences.
        """
        user = self.pool.get('res.users').browse(cr, uid, uid,
                                                 context=context)

        if not default:
            default = {}
        default.update({
            # affecté au commercial connecté qui duplique (et à sa company)
            'user_id': uid,  
            'company_id': user.company_id and user.company_id.id or False,
            'customer': True,
            'sap_client_code': False,
            'task_ids': False,
        })
        return super(
            res_partner,
            self).copy(
            cr,
            uid,
            id,
            default,
            context=context)
