# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from openerp.osv import osv, fields


class product_product(osv.Model):
    _inherit = 'product.product'
    
    def _get_fake(self, cr, uid, ids, name, args, context=None):
        res = {}
        for id in ids:
            res[id] = False
        return res
        
    def _get_user_product_category(self, cr, uid, context=None):
        # get user office code
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if not user or not user.company_id or not user.company_id.office_code:
            return False
        
        # get office product category from office code
        domain = [('office_code', '=', user.company_id.office_code)]
        categ_id = self.pool.get('product.category').search(cr, uid, domain,
            context=context)
        if not categ_id:
            return False
        return categ_id[0]
        
    def _of_company_search(self, cr, uid, ids, name, args, context=None):
        if not len(args):
            return [('id', '=', 0)]
        
        # get user product category
        categ_id = self._get_user_product_category(cr, uid, context=context)
        if not categ_id:
            return [('id', '=', 0)]
            
        # get products from category
        domain = [('categ_id', '=', categ_id)]
        ids = self.search(cr, uid, domain, context=context)
        if not ids:
            return [('id', '=', 0)]
        return [('id', 'in', ids)]
    
    _columns = {
        #'supply_ref': fields.char(u"Référence fournisseur", size=64, ),
        'sap_product_code': fields.char(u"Référence SAP", size=10, ),
        'axima_type': fields.char(u"Type interne", size=64, ),
        'import_flag': fields.char(u"Flag import", size=20, invisible=True),
        
        'of_company': fields.function(_get_fake, type='boolean',
            fnct_search= _of_company_search,
            string="De l'agence", method=True),
        }
        
    _defaults = {
        # By default, no partner is an instructor
        #'supply_ref': False,
        'sap_product_code': False,
        'axima_type': False,
        'import_flag': False,
    }

    def default_get(self, cr, uid, fields, context=None):
        res = super(product_product, self).default_get(cr, uid, fields,
            context=context)

        categ_id = self._get_user_product_category(cr, uid, context=context)
        if categ_id:
            res['categ_id'] = categ_id
        return res
    
    def _check_product_access(self, cr, uid, vals, ids, context=None):
        if uid == 1:
            return  # admin rights
        if 'import_category_products' in context:
            # import of category'product (create or write)
            return
        user = self.pool.get('res.users').browse(cr, uid, uid, context=context)
        if not user or not user.company_id:
            msg = u"Vous n'appartenez à aucune agence"
            raise osv.except_osv("Avertissement", msg)
        
        if ids:
            # from write
            
            # imported product ?
            imported = False
            products = self.browse(cr, uid, ids, context=context)
            for p in products:
                if p.import_flag and p.import_flag == 'imported':
                    imported = True
                    break
            if imported:
                msg = u"Vous n'êtes pas autorisé à modifier un" \
                    u" article de cette catégorie"
                raise osv.except_osv("Avertissement", msg)
            
            # product category must be of company
            categ_id = vals.get('categ_id', False)
            categ_ok = True
            if not categ_id:
                for p in products:
                    if p.categ_id and p.categ_id.office_code \
                        and p.categ_id.office_code != user.company_id.office_code:
                        categ_ok = False
                        break
            else:
                product_categ_r = self.pool.get('product.category').read(cr,
                    uid, categ_id, ['office_code'], context=context)
                if product_categ_r:
                    if not product_categ_r['office_code'] or (
                        product_categ_r['office_code'] and \
                        product_categ_r['office_code'] != user.company_id.office_code):
                        categ_ok = False
            if not categ_ok:
                tpl = u"Vous ne pouvez modifier qu'un article appartenant" \
                        u" à la catégorie d'article de votre agence '%s'"
                msg = tpl % (user.company_id.name, )
                raise osv.except_osv("Avertissement", msg)
        else:
            # from create              
            categ_id = vals.get('categ_id', False)
            if categ_id:
                product_categ_r = self.pool.get('product.category').read(cr,
                    uid, categ_id, ['office_code'], context=context)
                if product_categ_r:
                    if not product_categ_r['office_code'] or (
                        product_categ_r['office_code'] and \
                        product_categ_r['office_code'] != user.company_id.office_code):
                            tpl = u"Vous ne pouvez ajouter d'article que dans la" \
                                u" catégorie d'article de votre agence '%s'"
                            msg = tpl % (user.company_id.name, )
                            raise osv.except_osv("Avertissement", msg)
         
    def create(self, cr, uid, vals, context=None):
        self._check_product_access(cr, uid, vals, False, context=context)            
        return super(product_product, self).create(cr, uid, vals,
            context=context)
            
    def write(self, cr, uid, ids, vals, context=None):
        self._check_product_access(cr, uid, vals, ids, context=context)            
        return super(product_product, self).write(cr, uid, ids, vals,
            context=context)
            
    def unlink(self, cr, uid, ids, context=None):
        if isinstance(ids, (int, long)):
            ids = [ids]
        if uid != 1:  # admin can delete (deploy xmlrpc script, ...)
            # get user product category
            categ_id = self._get_user_product_category(cr, uid, context=context)
            if categ_id:
                r_products = self.read(cr, uid, ids, ['categ_id'], context=context)
                for rp in r_products:
                    if rp['categ_id'][0] and rp['categ_id'][0] != categ_id:
                        msg = u"Vous pouvez uniquement supprimer des articles de votre catégorie d'agence"
                        raise osv.except_osv("Avertissement", msg)                
        return super(product_product, self).unlink(cr, uid, ids,
            context=context)
            
    def copy(self, cr, uid, id, default=None, context=None):
        """
        set product category to user company
        """
        user = self.pool.get('res.users').browse(cr, uid, uid,
                                                 context=context)
        if user.company_id and user.company_id.name:
            domain = [('office_code', '=', user.company_id.office_code)]
            categ_id = self.pool.get('product.category').search(cr, uid, domain,
                context=context)
            if categ_id:
                categ_id = categ_id[0]

        if not default:
            default = {}
        default.update({
            'import_flag': False,
        })
        if categ_id:
            default['categ_id'] = categ_id
        return super(
            product_product,
            self).copy(
            cr,
            uid,
            id,
            default,
            context=context)
            
 
class product_category(osv.Model):
    _inherit = 'product.category'

    _columns = {
        'sap_company_code': fields.char(u"Référence SAP", size=10, ),
        'office_code': fields.char(u"Code agence", size=10, ),
    }
    
    def create(self, cr, uid, vals, context=None):
        if uid != 1:  # admin can create (deploy xmlrpc script, ...)
            msg = u"Vous ne pouvez pas créer de catégorie d'article"
            raise osv.except_osv("Avertissement", msg)          
        return super(product_category, self).create(cr, uid, vals,
            context=context)
            
    def write(self, cr, uid, ids, vals, context=None):
        if uid != 1:  # admin can write (deploy xmlrpc script, ...)
            msg = u"Vous ne pouvez pas modifier de catégorie d'article"
            raise osv.except_osv("Avertissement", msg)            
        return super(product_category, self).write(cr, uid, ids, vals,
            context=context)
            
    def unlink(self, cr, uid, ids, context=None):
        if uid != 1:  # admin can delete (deploy xmlrpc script, ...)
            msg = u"Vous ne pouvez pas supprimer de catégorie d'article"
            raise osv.except_osv("Avertissement", msg)            
        return super(product_category, self).unlink(cr, uid, ids,
            context=context)
