# -*- coding: utf-8 -*-
##############################################################################
#
#    Original Module for OpenERP, Open Source Management Solution
#    Copyright (C) 20014 TeMPO CONSULTING (<http://tempo-consulting.fr>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import fields, osv


class sale_order_line(osv.osv):
    _inherit = "sale.order.line"

    def _purchase_sum(self, cr, uid, ids, field_name, arg, context=None):
        """Compute the purchase price subtotal of lines"""
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            qty = line.product_uos_qty
            if line.product_id:
                if line.purchase_price:
                    bought_price = (line.purchase_price*qty)
                else:
                    bought_price = (line.product_id.standard_price * qty)
                try:
                    res[line.id] = bought_price
                except UnboundLocalError:
                    res[line.id] = 0.0
        return res

    def _sale_sum(self, cr, uid, ids, field_name, arg, context=None):
        """Compute the sale price subtotal of lines"""
        res = {}
        for line in self.browse(cr, uid, ids, context=context):
            qty = line.product_uos_qty
            sale_price = (
                line.price_unit*qty * (100.0-line.discount)/100.0) or 0
            res[line.id] = sale_price
        return res

    _columns = {
        'purchase_sum': fields.function(_purchase_sum,
            string="Sous-total du prix d'achat",
            store=True),

        'sale_sum': fields.function(_sale_sum,
            string="Sous-total du prix de vente",
            store=True),
    }

sale_order_line()


class sale_order(osv.osv):
    _inherit = "sale.order"

    def _product_margin_rate(self, cr, uid, ids, field_name, arg,
        context=None):
        result = {}
        for sale in self.browse(cr, uid, ids, context=context):
            sale_price = sale.amount_untaxed or 0.0
            # remise global, calcul du taux de marge ce fait sur le
            # total ht non remisé
            # (par contre la marge brute (montant) déduit la remise globale)
            if sale.global_discount and sale.global_discount > 0.:
                sale_price += sale.global_discount
            if sale_price == 0.0:
                result[sale.id] = 0.0
            else:
                result[sale.id] = round((sale.margin / sale_price) * 100, 2)
        return result
        
    def _product_margin(self, cr, uid, ids, field_name, arg, context=None):
        result = {}
        for sale in self.browse(cr, uid, ids, context=context):
            result[sale.id] = 0.0
            for line in sale.order_line:
                result[sale.id] += line.margin or 0.0
            # deduire la remise globale éventuelle de la marge
            if sale.global_discount and sale.global_discount > 0.:
                result[sale.id] -= sale.global_discount
        return result

    def _get_order(self, cr, uid, ids, context=None):
        result = {}
        for line in self.pool.get('sale.order.line').browse(cr, uid, ids, context=context):
            result[line.order_id.id] = True
        return result.keys()
        
    def _get_kg(self, cr, uid, ids, field_name, arg, context=None):
        """
        Calcul du KG: Coefficient Somme(prix vente) / Somme(prix achat)
        """
        result = {}
        for sale in self.browse(cr, uid, ids, context=context):
            sale_price = sale.amount_untaxed or 0.
            pur_price = 0
            for sol in sale.order_line:
                pur_price += sol.purchase_price * sol.product_uom_qty
            result[sale.id] = round((sale_price / pur_price), 2) if pur_price else 0.
        return result

    _margin_rate_help = (u"Fourni le taux de rentabilité : "
                         u"(prix de vente - prix d’achat)/prix de vente.")
    _columns = {
        'margin_rate': fields.function(_product_margin_rate, type='float',
            string='Taux de marge', help=_margin_rate_help,
            method=True),
        'margin': fields.function(_product_margin, string='Margin', help="It gives profitability by calculating the difference between the Unit Price and the cost price.",
            store={
                'sale.order.line': (_get_order, ['order_id', 'margin'], 20),  # AX-23: trigger de order_id le module nan_product_pack writtant directement des sale.order.line associé à un order_id
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'global_discount'], 20),
            }),
        'kg': fields.function(_get_kg, type='float',
            string='KG', help=u"Coefficient Somme(prix vente) / Somme(prix achat)",
            method=True),
    }

sale_order()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
