# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from openerp.osv import osv
from openerp.osv import fields

import openerp.addons.decimal_precision as dp


class sale_order(osv.osv):
    _inherit = 'sale.order'

    def _amount_all(self, cr, uid, ids, field_name, arg, context=None):
        """
        Update the context to take care of the discount
        """
        discount_tax_obj = self.pool.get('sale.order.discount.line')

        if context is None:
            context = {}

        res = super(sale_order, self)._amount_all(cr, uid, ids, field_name, arg, context=context)

        for order in self.browse(cr, uid, ids, context=context):
            res[order.id].update({
                'amount_untaxed': res[order.id]['amount_untaxed'] - order.global_discount,
                'amount_total': res[order.id]['amount_total'] - order.global_discount,
            })

            discount_lines = discount_tax_obj.search(cr, uid, [('order_id', '=', order.id)], context=context)
            for disc_line in discount_tax_obj.browse(cr, uid, discount_lines, context=context):
                res[order.id]['amount_tax'] -= disc_line.discount
                res[order.id]['amount_total'] -= disc_line.discount

        return res

    def _get_order(self, cr, uid, ids, context=None):
        """
        Update the context to take care of the discount
        """
        sol_obj = self.pool.get('sale.order.line')

        if context is None:
            context = {}

        res = {}

        for line in sol_obj.browse(cr, uid, ids, context=context):
            res[line.order_id.id] = True

        return res

    def _get_lines_total_amount(self, cr, uid, ids, context=None):
        """
        :return : { 'id', lines_total, ... }
        """
        res = {}
        if not ids:
            return res
        if isinstance(ids, (int, long)):
            ids = [ids]
        for so in self.browse(cr, uid, ids, context=context):
            amount = 0.
            for sl in so.order_line:
                if sl.price_subtotal:
                    amount += sl.price_subtotal
            res[so.id] = amount
        return res

    _columns = {
        'global_discount': fields.float(
            string='Remise globale',
            digits_compute=dp.get_precision('Account'),
        ),
        'global_discount_rate': fields.float('Remise globale', ),
        'amount_untaxed': fields.function(
            _amount_all,
            digits_compute=dp.get_precision('Account'),
            string='Untaxed Amount',
            store={
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'global_discount'], 10),
                'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
            },
            multi='sums',
            help="The amount without tax.",
            track_visibility='always',
        ),
        'amount_tax': fields.function(
            _amount_all,
            digits_compute=dp.get_precision('Account'),
            string='Taxes',
            store={
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'global_discount'], 10),
                'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
            },
            multi='sums',
            help='The tax amount',
        ),
        'amount_total': fields.function(
            _amount_all,
            digits_compute=dp.get_precision('Account'),
            string='Total',
            store={
                'sale.order': (lambda self, cr, uid, ids, c={}: ids, ['order_line', 'global_discount'], 10),
                'sale.order.line': (_get_order, ['price_unit', 'tax_id', 'discount', 'product_uom_qty'], 10),
            },
            multi='sums',
            help='The total amount.',
        ),
    }

    def _compute_discount(self, cr, uid, ids, vals, context=None):
        """
        """
        discount_tax_obj = self.pool.get('sale.order.discount.line')
        tax_obj = self.pool.get('account.tax')

        if context is None:
            context =  {}

        if isinstance(ids, (int, long)):
            ids = [ids]

        for order_id in ids:
            if vals and 'global_discount' in vals:
                order_brw = self.browse(cr, uid, order_id, context=context)

                # Get all taxes used in sale.order object
                discount_tax = {}
                for line in order_brw.order_line:
                    nb_tax = len(line.tax_id)
                    for tax in line.tax_id:
                        discount_tax.setdefault(tax, 0.00)
                        discount_tax[tax] += (line.price_unit * (1 - (line.discount or 0.0) / 100.0)) / float(nb_tax) * line.product_uom_qty

                # Get all existing discount lines for this order
                discount_meet = {}
                discount_order_ids = discount_tax_obj.search(cr, uid, [('order_id', '=', order_brw.id)], context=context)
                for disc in discount_tax_obj.browse(cr, uid, discount_order_ids, context=context):
                    discount_meet.setdefault(disc.tax_id, disc['id'])

                # Update or create discount lines
                for tax, tax_total in discount_tax.iteritems():
                    discount = 0.00
                    if order_brw.amount_untaxed or order_brw.global_discount:
                        discount = tax_obj.compute_all(
                            cr,
                            uid,
                            [tax],
                            (tax_total / (order_brw.amount_untaxed + order_brw.global_discount) * vals['global_discount']),
                            1,
                        )['taxes'][0]['amount']
                    discount_vals = {
                        'order_id': order_brw.id,
                        'tax_id': tax.id,
                        'discount': discount
                    }

                    if tax in discount_meet.keys():
                        discount_tax_obj.write(cr, uid, [discount_meet[tax]], discount_vals, context=context)
                    else:
                        discount_tax_obj.create(cr, uid, discount_vals, context=context)

                # Put 0.00 as discount for discount lines with a tax not used in sale.order object
                for tax_key in discount_meet.keys():
                    if tax_key not in discount_tax:
                        discount_tax_obj.write(cr, uid, [discount_meet[tax_key]], {'discount': 0.00}, context=context)

        return True

    def create(self, cr, uid, vals, context=None):
        """
        """
        order_id = super(sale_order, self).create(cr, uid, vals, context=context)

        self._compute_discount(cr, uid, order_id, vals, context=context)
        
        return order_id

    def write(self, cr, uid, ids, vals=False, context=None):
        """
        """
        self._compute_discount(cr, uid, ids, vals, context=context)

        return super(sale_order, self).write(cr, uid, ids, vals, context=context)

    def recompute_amounts(self, cr, uid, ids, context=None):
        """
        Re-compute the amounts
        """
        if isinstance(ids, (int, long)):
            ids = [ids]

        for order in self.read(cr, uid, ids, ['global_discount'], context=context):
            disc = order['global_discount']
            self.write(cr, uid, [order['id']], {'global_discount': disc}, context=context)

        return True

    def action_button_confirm(self, cr, uid, ids, context=None):
        """
        Re-compute the amounts
        """
        self.recompute_amounts(cr, uid, ids, context=context)

        return super(sale_order, self).action_button_confirm(cr, uid, ids, context=context)

    def button_dummy(self, cr, uid, ids, context=None):
        """
        Re-compute the amounts
        """
        self.recompute_amounts(cr, uid, ids, context=context)

        return super(sale_order, self).button_dummy(cr, uid, ids, context=context)

    def on_change_global_discount_rate(self, cr, uid, ids, global_discount_rate,
        context=None):
        """
        from a user input of global discount rate, compute the global rate
        amount truely used by the global amount system
        """
        if not ids:
            return {}
        if isinstance(ids, (int, long)):
            ids = [ids]
        lines_total = self._get_lines_total_amount(cr, uid, ids,
            context=context)[ids[0]]  # get so lines total to compute from

        discount_amount = 0
        if global_discount_rate and \
            isinstance(global_discount_rate, (str, unicode)):
            # % widget conversion 2 float
            global_discount_rate = float(global_discount_rate.replace(',', '.'))
        if not global_discount_rate:
            global_discount_rate = 0.
        if lines_total and lines_total > 0.:
            discount_amount = (lines_total * global_discount_rate) / 100
        err = False
        if discount_amount > lines_total:
            err = 'La remise globale ne peut dépasser le montant'
        elif discount_amount < 0:
            err = 'La remise globale ne peut être négative'
        if err:
            res = {
                'value': {'global_discount_rate': 0., 'global_discount': 0.,},
                'warning': {
                    'title': 'Remise globale',
                    'message': err,
                },
            }
        else:
            res = {
                'value': {'global_discount': discount_amount, },
            }
        return res


class sale_order_discount_line(osv.osv):
    _name = 'sale.order.discount.line'
    _description = 'Discount line for sale order'

    _columns = {
        'order_id': fields.many2one(
            'sale.order',
            string='Order ref.',
            required=True,
            readonly=True,
            ondelete='cascade',
        ),
        'tax_id': fields.many2one(
            'account.tax',
            string='Tax ref.',
            required=True,
            readonly=True,
        ),
        'discount': fields.float(
            string='Discount',
            digits_compute=dp.get_precision('Account'),
            readonly=True,
        ),
    }


class sale_order_line(osv.osv):
    _inherit = 'sale.order.line'

    def create(self, cr, uid, vals, context=None):
        """
        Re-compute the global discount of the order when a line is added.
        """
        res = super(sale_order_line, self).create(cr, uid, vals, context=context)

        if vals.get('order_id'):
            self.update_order_discount(cr, uid, vals.get('order_id'), context=context)

        return res

    def write(self, cr, uid, ids, vals, context=None):
        """
        Re-compute the global discount of the order when a line is modified.
        """
        res = super(sale_order_line, self).write(cr, uid, ids, vals, context=context)

        if vals.get('price_unit') or vals.get('tax_id') or vals.get('product_qty') or vals.get('discount'):
            order_ids = set()
            if vals.get('order_id'):
                order_ids.add(vals.get('order_id'))
            else:
                for line in self.browse(cr, uid, ids, context=context):
                    order_ids.add(line.order_id.id)

            self.update_order_discount(cr, uid, list(order_ids), context=context)

        return res

    def unlink(self, cr, uid, ids, context=None):
        """
        Re-compute the global discount of the order when a line is removed.
        """
        order_ids = set()

        if isinstance(ids, (int, long)):
            ids = [ids]

        for line in self.browse(cr, uid, ids, context=context):
            order_ids.add(line.order_id.id)

        res = super(sale_order_line, self).unlink(cr, uid, ids, context=context)

        self.update_order_discount(cr, uid, list(order_ids), context=context)

        return res

    def update_order_discount(self, cr, uid, order_ids, context=None):
        """
        Re-compute the global discount of the order
        """
        order_obj = self.pool.get('sale.order')

        if context is None:
            context = {}

        if isinstance(order_ids, (int, long)):
            order_ids = [order_ids]

        if not order_ids:
            return True

        for order in order_obj.read(cr, uid, order_ids, ['global_discount'], context=context):
            order_obj._compute_discount(cr, uid, order['id'], {
                'global_discount': order['global_discount'],
            }, context=context)

        return True
